<?php

/**
 * BuddyServ IRC Services
 * 
 * Language holder class. Maintains the different
 * language options for all members.
 * 
 * @name Language
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.language.php 350 2009-12-03 07:55:58Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class language {
	
	public static $available = array();
	public static $helpfiles = array();
	public $short;
	public $using = false;
	public $service_help = false;
	public static $default;
	public static $geo = array();
	public static $short_long = array();
	
	/**
	 * Populate the available languages, and
	 * store them in memory.
	 * 
	 * @return void
	 */
	public static function populate()
	{
		global $config;
		
		self::$geo = array(
			'CA'	=> 'english',
			'US'	=> 'english',
			'FR'	=> 'francais',
			'ES'	=> 'espanol',
			'AU'	=> 'english',
			'GB'	=> 'english'
		);
		
		self::$short_long = array(
			'EN'	=> 'english',
			'FR'	=> 'francais',
			'ES'	=> 'espanol',
			'AU'	=> 'english',
			'GB'	=> 'english'
		);
		
		/* cycle the config list of available languages */
		self::$default = strtolower($config['languages']['default']);
		
		foreach ($config['languages']['available'] as $each => $enabled)
		{
			$namelower = strtolower($each);
			
			/* set to false in config */
			if (!$enabled)
				continue;
				
			/* check if the strings file exists for this language */
			if (!file_exists('languages/'.$namelower.'/strings.php'))
				continue;
				
			require_once('languages/'.$namelower.'/strings.php');
			$array = ${$namelower};
			
			self::$available[$namelower] = $array;
			
			/* Load the help files for this language */
			self::loadHelp('chanserv', $namelower);
			self::loadHelp('nickserv', $namelower);
			self::loadHelp('memoserv', $namelower);
			self::loadHelp('userbot', $namelower);
			self::loadHelp('operserv', $namelower);
		}
		
	}
	
	/**
	 * Check if the given service help topic exists.
	 * 
	 * @param string $service
	 * @param string $topic
	 * @return bool
	 */
	public function helpTopicExists($service, $topic)
	{
		if (array_key_exists($topic, $this->service_help[$service]))
			return true;

		return false;
	}
	
	/**
	 * Returns a service help topic as an array.
	 * Returns false if not existing.
	 * 
	 * @param string $service
	 * @param string $topic
	 * @return array|false
	 */
	public function getHelpTopic($service, $topic)
	{
		if (!$this->helpTopicExists($service, $topic))
			return false;
			
		return $this->service_help[$service][$topic];
	}
	
	/**
	 * Load the service help file for the given
	 * language, and store it in the array.
	 * 
	 * @param string $service
	 * @param string $lang
	 * @return void
	 */
	public static function loadHelp($service, $lang)
	{
		self::$helpfiles[$lang][$service] = array();
		
		/* check if we have a valid help directory, if so, read the file listing */
		if (is_dir('languages/'.$lang.'/help/'.$service))
		{
				if ($handle = opendir('languages/'.$lang.'/help/'.$service))
				{
					while (($file = readdir($handle)) !== false)
					{
						/* we never really want to recursively search, and ../ is bad */
						if (filetype('languages/'.$lang.'/help/'.$service.'/'.$file) == 'dir')
							continue;
							
						/* if the file is an empty placeholder dont include it */
						if (filesize('languages/'.$lang.'/help/'.$service.'/'.$file) == 0)
						{
							continue;
						}
						
						/* open the file read only */
						$fhandle = fopen('languages/'.$lang.'/help/'.$service.'/'.$file, 'r');
						
						$buffer = fread($fhandle, filesize('languages/'.$lang.'/help/'.$service.'/'.$file));
						fclose($fhandle);
						
						/* Remove the file extension from the name  */
						$filestoreas = substr($file, 0, (strlen($file) - 4));
						
						self::$helpfiles[$lang][$service][$filestoreas] = explode("\r\n", $buffer);
					}
					
				closedir($handle);
			}
		}
	}
	
	/**
	 * Magic get function to return a language string.
	 * 
	 * @param string $what
	 * @return string
	 */
	public function __get($what)
	{
		return $this->string($what);
	}
	
	/**
	 * Construct our language object.
	 * This uses references to help save
	 * memory.
	 * 
	 * @param string $language
	 * @return language
	 */
	public function __construct($language)
	{
		$lower = strtolower($language);
		
		/* Check if this language has been loaded,
		 * if not, default to.. well, our default..
		 * 
		 * If our default doesn't exist.. send an
		 * operwall warning about it and.. well, try
		 * and fall back to english.
		 */
		if (!array_key_exists($lower, language::$available))
		{
			/* what about default? */
			if (!array_key_exists(language::$default, language::$available))
			{
				/* Operwall the fact that our default language settings do
				 * NOT exist. This isn't good.
				 */
				operserv::$client->operwall('Warning! - Default language not found: '.language::$default);
				
				/* check for english */
				if (array_key_exists('english', language::$available))
				{
					$this->using = &language::$available['english'];
					$this->short = 'EN';
					$this->service_help = &language::$helpfiles['english'];
				}
				
				else
				{
					/* what? where is our english language files? We should NEVER hit this.
					 * If english doesnt exist, and neither does the default, services should
					 * have failed to load, but just incase, terminate services after an operwall.
					 */
					operserv::$client->operwall('Fatal: Unable to find fallback English dictionary file. Goodbye.');
					die('Failed to find english language files, or default language files.');
				}
			}
			
			/* The default language requested exists; the requested language does not. */
			else
			{
				$this->using = &language::$default;
				$this->service_help =  &language::$helpfiles[language::$default];
				
				foreach (language::$short_long as $sl => $lang)
				{
					if (!strcasecmp($lang, language::$default))
						$this->short = $sl;
				}
			}
				
		}
		
		/* The requested language exists. */
		else
		{
			$this->using = &language::$available[$lower];
			$this->service_help =  &language::$helpfiles[$lower];
			
			foreach (language::$short_long as $sl => $lang)
			{
				if (!strcasecmp($lang, $lower))
					$this->short = $sl;
			}
		}
	}
	
	/**
	 * Check if the language given exists, and is active
	 * 
	 * @param string $lang
	 * @return bool
	 */
	public static function exists($lang)
	{
		if (array_key_exists(strtolower($lang), self::$available))
			return true;
			
		return false;
	}
	
	/**
	 * Returns the given language string
	 * 
	 * @param string $what
	 * @return string
	 */
	public function string($what)
	{
		if (array_key_exists($what, $this->using))
			return $this->using[$what];
			
		return null;
	}
}

	

?>