<?php

/**
 * BuddyServ IRC Services
 * 
 * ServiceClient object used by all pseudo
 * service clients. Provides common functionality
 * to the IRC Server.
 * 
 * @name ServiceClient
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.serviceclient.php 344 2009-11-29 07:13:34Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class serviceclient
{
	public $irc;
	public $username;
	public $host;
	public $name;
	public $nick;
	public $modes;
	public $help_list = array();
	
	/**
	 * Initialize the generic model
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	public function initialize($nick, $username, $host, $name, $modes)
	{
		global $irc;
		
		$this->nick = $nick;
		$this->host = $host;
		$this->username = $username;
		$this->name = $name;
		$this->modes = $modes;
		$this->irc = &$irc;
	}
	
	/**
	 * Introduce the client to the network
	 * 
	 * @return void
	 */
	public function introduce()
	{
		global $me;
		
		if (!$this->nick)
			die('Call to introduce() on non initialized service client');
			
		$this->irc->send(sprintf(INTRODUCE, $this->nick, 
			1, 
			time(), 
			$this->modes, 
			$this->username, 
			$this->host, 
			$me->name, 
			0, 
			$me->name, 
			$this->name));
	}
	
	/**
	 * Kill a user off the network
	 * 
	 * @param string $target
	 * @param string $reason
	 * @return void
	 */
	public function kill($target, $reason)
	{
		global $me;
		
		$this->irc->send(sprintf(KILL, 
			$this->nick, 
			$target, 
			$me->name, 
			$this->host, 
			$this->username,
			$me->name,
			$reason));
								
		user::remuser($target);
	}
	/**
	 * Sends an IRC Notice
	 * 
	 * @param string $target
	 * @param string $message
	 * @return void
	 */
	public function notice($target, $message)
	{
		$this->irc->send(sprintf(NOTICE, $this->nick, $target, $message));
	}
	
	/**
	 * Sets a channels modes
	 * 
	 * @param string $target
	 * @param string $modes
	 * @return unknown_type
	 */
	public function chanMode($target, $modes)
	{
		$this->irc->send(sprintf(MODE, $this->nick, $target, $modes));
		handler::cMode(array(null, false, $target, $modes));
	}
	
	/**
	 * Show an 'Access Denied' string to a client.
	 * 
	 * @param string $target
	 * @return void
	 */
	public function denied($target)
	{
		self::notice($target, 'Access denied.');
	}
	
	/**
	 * Sends an operwall
	 * 
	 * @param string $message
	 * @return void
	 */
	public function operwall($message)
	{
		$this->irc->send(sprintf(OPERWALL, $this->nick, $message));
	}
	
	/**
	 * Join a channel
	 * 
	 * @param string $channel
	 * @return void
	 */
	public function join($channel)
	{
		global $me;
		$this->irc->send(sprintf(SJOIN, $me->name, time(), $channel, '+', $this->nick));
	}
	
	/**
	 * Set a channel mode. Legacy place holder.
	 * 
	 * @param string $channel
	 * @param string $modes
	 * @return void
	 */
	public function cmode($channel,$modes)
	{
		self::chanMode($channel, $modes);
	}
	
	/**
	 * Kick a user from a channel
	 * 
	 * @param string $channel
	 * @param string $who
	 * @param string $reason
	 * @return void
	 */
	public function kick($channel, $who, $reason='No reason specified.')
	{
		global $me;
		$this->irc->send(sprintf(KICK, $this->nick, $channel, $who, $reason));
		
		$userObj = user::getuser($who);
		$userObj->removeChannel($channel);
		
		$chanObj = channel::get($channel);
		$chanObj->deluser($who);
		
		if (!strcasecmp($channel, $me->user_channel))
			idle::remove($who, true);
	}
	
	/**
	 * Change a channel topic
	 * 
	 * @param string $channel
	 * @param string $topic
	 * @return void
	 */
	public function topic($channel, $topic)
	{
		$this->irc->send(sprintf(TOPIC, $this->nick, $channel, $topic));
	}
	
	/**
	 * Part a channel
	 * 
	 * @param string $channel
	 * @param string $reason
	 * @return void
	 */
	public function part($channel, $reason=false)
	{
		$this->irc->send(sprintf(PART, $this->nick, $channel, ($reason ? $reason : '')));
	}
	
	/**
	 * Quits the current service client
	 * 
	 * @param string $reason
	 * @return void
	 */
	public function quit($reason='BuddyServ by Lynk')
	{
		$this->irc->send(sprintf(QUIT, $this->nick, $reason));
	}
			
	/**
	 * Send a PRIVMSG
	 * 
	 * @param string $target
	 * @param string $message
	 * @return void
	 */
	public function msg($target, $message)
	{
		$this->irc->send(sprintf(MSG, $this->nick, $target, $message));
	}
	
	
	/**
	 * Handle help requests for the service client.
	 * 
	 * @param string $nick
	 * @param string $service
	 * @param string $arguments
	 * @return void
	 */
	public function help($nick, $service, $arguments=false)
	{
		$user = user::getuser($nick);
		
		if (!$arguments)
		{
			if ($user->language->helpTopicExists($service, 'index'))
			{
				$array = $user->language->getHelpTopic($service, 'index');
			
				foreach ($array as $number => $line)
				{
					$this->notice($nick, $line);
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_UPLOADER))
			{
				if ($user->language->helpTopicExists($service, 'oindex'))
				{
					$array = $user->language->getHelpTopic($service, 'oindex');
					
					foreach ($array as $number => $line)
					{
						$this->notice($nick, $line);
					}
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_MODERATOR))
			{
				if ($user->language->helpTopicExists($service, 'aindex'))
				{
					$array = $user->language->getHelpTopic($service, 'aindex');
					
					foreach ($array as $number => $line)
					{
						$this->notice($nick, $line);
					}
				}
			}
			
		} /* we have arguments after this */
		
		else 
		{	
			if (count($arguments) > 1)
			{
				$type = strtolower(implode('_', $arguments));
			}
			
			else
			{
				$type = strtolower($arguments[0]);
			}

			if ($user->language->helpTopicExists($service, $type))
			{
				$array = $user->language->getHelpTopic($service, $type);
				
				foreach ($array as $number => $line)
				{
					$this->notice($nick, $line);
				}
				
				return;
			}
				
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_UPLOADER))
			{
				$tkey = 'o'.$type;
				
				if ($user->language->helpTopicExists($service, $tkey))
				{
					$array = $user->language->getHelpTopic($service, $tkey);
					
					foreach ($array as $number => $line)
					{
						$this->notice($nick, $line);
					}
					
					return;
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_ADMIN))
			{
				$tkey = 'a'.$type;
				
				if ($user->language->helpTopicExists($service, $tkey))
				{
					$array = $user->language->getHelpTopic($service, $tkey);
					
					foreach ($array as $number => $line)
					{
						$this->notice($nick, $line);
					}
					
					return;
				}
			}
			
			/* unknown topix */
			$this->notice($nick, 'No such help topic.');
		}
	}
	
}

?>
	